@@ -60,6 +60,7 @@ namespace Python
 
 DeclarationBuilder::DeclarationBuilder(PythonEditorIntegrator* editor)
         : DeclarationBuilderBase( )
+        , m_ownPriority(0)
 {
     setEditor(editor);
     kDebug() << "Building Declarations";
@@ -88,6 +89,7 @@ ReferencedTopDUContext DeclarationBuilder::build(const IndexedString& url, Ast*
     if ( ! m_prebuilding ) {
         kDebug() << "building, but running pre-builder first";
         DeclarationBuilder* prebuilder = new DeclarationBuilder(editor());
+        prebuilder->m_ownPriority = m_ownPriority;
         prebuilder->m_currentlyParsedDocument = currentlyParsedDocument();
         prebuilder->setPrebuilding(true);
         prebuilder->m_futureModificationRevision = m_futureModificationRevision;
@@ -812,15 +814,30 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
         return 0;
     }
     if ( ! moduleContext ) {
+        KUrl modulePath = moduleInfo.first;
         // schedule the include file for parsing, and schedule the current one for reparsing after that is done
         kDebug() << "No module context, recompiling";
-        m_unresolvedImports.append(moduleInfo.first);
-        if ( KDevelop::ICore::self()->languageController()->backgroundParser()->isQueued(moduleInfo.first) ) {
-            KDevelop::ICore::self()->languageController()->backgroundParser()->removeDocument(moduleInfo.first);
+        m_unresolvedImports.append(modulePath);
+        BackgroundParser* bgparser = KDevelop::ICore::self()->languageController()->backgroundParser();
+        bool needsReschedule = true;
+        if ( bgparser->isQueued(modulePath) ) {
+            const ParseJob* job = bgparser->parseJobForDocument(modulePath);
+            int previousPriority = BackgroundParser::WorstPriority;
+            if ( job ) {
+                previousPriority = job->parsePriority();
+            }
+            // if it's less important, reschedule it
+            if ( job && previousPriority > m_ownPriority - 1 ) {
+                bgparser->removeDocument(modulePath);
+            }
+            else if ( job ) {
+                needsReschedule = false;
+            }
+        }
+        if ( needsReschedule ) {
+            bgparser->addDocument(modulePath, TopDUContext::ForceUpdate, m_ownPriority - 1,
+                                  0, ParseJob::FullSequentialProcessing);
         }
-        KDevelop::ICore::self()->languageController()->backgroundParser()
-                                   ->addDocument(moduleInfo.first, TopDUContext::ForceUpdate, m_ownPriority - 1,
-                                                 0, ParseJob::FullSequentialProcessing);
         // parseDocuments() must *not* be called from a background thread!
         // KDevelop::ICore::self()->languageController()->backgroundParser()->parseDocuments();
         return 0;
@@ -1037,14 +1054,13 @@ void DeclarationBuilder::visitCall(CallAst* node)
             bool atVararg = false;
             // Check that there's enough known parameters which can be updated
             uint typeParametersSize = functiontype->arguments().length();
-            typeParametersSize += static_cast<FunctionDeclarationPointer>(lastFunctionDeclaration)->defaultParametersSize();
             if ( parameters.size() >= node->arguments.size() && typeParametersSize >= (uint) node->arguments.size() ) {
                 foreach ( ExpressionAst* arg, node->arguments ) {
                     // Iterate over all the arguments, trying to guess the type of the object being
                     // passed as an argument, and update the parameter accordingly.
                     
                     // If more params are passed than the function has args, maybe it's a vararg
-                    if ( atParam >= functiontype->arguments().size() - specialParamsCount || atParam >= parameters.size() - specialParamsCount ) {
+                    if ( atParam >= functiontype->arguments().size() - specialParamsCount && atParam >= parameters.size() - specialParamsCount ) {
                         if ( lastFunctionDeclaration->hasVararg() ) {
                             atVararg = true;
                         }
@@ -1679,13 +1695,15 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
                                                                   <AbstractType>("string", currentContext());
                 VariableLengthContainer::Ptr dictType = ExpressionVisitor::typeObjectForIntegralType
                                                                   <VariableLengthContainer>("dict", currentContext());
-                lock.unlock();
-                dictType->addKeyType(stringType);
-                node->kwarg->startCol = node->arg_col_offset; node->kwarg->endCol = node->arg_col_offset + node->kwarg->value.length() - 1;
-                node->kwarg->startLine = node->arg_lineno; node->kwarg->endLine = node->arg_lineno;
-                visitVariableDeclaration<Declaration>(node->kwarg, 0, dictType.cast<AbstractType>());
-                workingOnDeclaration->setHasKwarg(true);
-                type->addArgument(dictType.cast<AbstractType>());
+                if ( dictType && stringType ) {
+                    lock.unlock();
+                    dictType->addKeyType(stringType);
+                    node->kwarg->startCol = node->arg_col_offset; node->kwarg->endCol = node->arg_col_offset + node->kwarg->value.length() - 1;
+                    node->kwarg->startLine = node->arg_lineno; node->kwarg->endLine = node->arg_lineno;
+                    visitVariableDeclaration<Declaration>(node->kwarg, 0, dictType.cast<AbstractType>());
+                    workingOnDeclaration->setHasKwarg(true);
+                    type->addArgument(dictType.cast<AbstractType>());
+                }
             }
         }
     }
